%% Setup
close all
clear
clc

% Author: Christian Thode Larsen, christian.thode.larsen@gmail.com


%% setup tests

%runType = 'long';
%runType = 'EM3G';
% runType = 'EM6G';
%runType = 'N3';
% runType = 'sameVariance';
%runType = 'sameVarianceFreeMeans';
%runType = 'mixtureWithPrior';
runType = 'EMwithSV';

showPlots = true;

%% some environment dependent variables - configure as needed..


addpath '/opt/freesurfer/stable/matlab'
basePath = '/home/cthla/Documents/phd/trunk/projects/bias/dev/data';
dataPath = [basePath, '/ADNI_PrepPhase/007-S-0002-scan1.mgz'];
data2Path = [basePath, '/ADNI_PrepPhase/007-S-0002-scan2.mgz'];
% NOTE! Prior is coregistered and resampled already
priorWMPath = [basePath, '/ADNI_PrepPhase/priors/007-S-0002-scan1-white.mgz'];
priorGMPath = [basePath, '/ADNI_PrepPhase/priors/007-S-0002-scan1-grey.mgz'];
priorCSFPath = [basePath, '/ADNI_PrepPhase/priors/007-S-0002-scan1-csf.mgz'];

dataS = MRIread(dataPath);
data2S = MRIread(data2Path);
priorWM = MRIread(priorWMPath);
priorGM = MRIread(priorGMPath);
priorCSF = MRIread(priorCSFPath);

% prior is a 2D array, voxels x classes
prior = [priorWM.vol(:), priorGM.vol(:), priorCSF.vol(:)];

%     addpath '/usr/local/freesurfer/stable/matlab'
%     basePath = '/cluster/koen/christian/dev';
%     %     dataPath = [basePath, '/data/7T-MEMPRAGE-750/MEMPRAGE_2e_p2_750iso_FOCI_500V__2014_11_14.mgz'];
%     %     maskPath = [basePath, '/tests/7T-MEMPRAGE-750/MASK/MEMPRAGE_2e_p2_750iso_FOCI_500V__2014_11_14-fsmask-eroded0.mgz'];
%     dataPath = [basePath, '/data/scan-rescan-T1/sub02-T1-1.mgz'];
%     maskPath = [basePath, '/tests/scan-rescan-T1/MASK/sub02-T1-1-fsmask-eroded0.mgz'];
%
%     %dataPath = [basePath, '/scan-rescan-T1/sub02-T1-1.mgz'];
%
%     dataS = MRIread(dataPath);
%     maskS = MRIread(maskPath);


switch runType
    case 'long'
        data = ones([size(dataS.vol), 2]);
        data(:,:,:,1) = dataS.vol;
        data(:,:,:,2) = data2S.vol;
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(data, ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'backgroundThresholdType', 'otzu', ...
            'numberOfGaussians', 6, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
    case 'EM6G'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            ...%            'mask', maskS.vol, ...
            'backgroundThresholdType', 'otzu', ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'numberOfGaussians', 6, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 861e-5, ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
    case 'EM3G'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            ...%          'mask', maskS.vol, ...
            'backgroundThresholdType', 'otzu', ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'numberOfGaussians', 3, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 861e-5, ...
            'backgroundThresholdType', 'otzu', ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
    case 'N3'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            'backgroundThresholdType', 'otzu', ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 3, ...
            'mimicN3', 'model', ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
    case 'sameVariance'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            'backgroundThresholdType', 'otzu' , ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'numberOfGaussians', 24, ...
            'sameVariance', true, ...
            'varianceScale', 1, ...
            'updateVariance', true, ...
            'equidistantMeans', true, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'smoothingType', 'bspline', ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
    case 'sameVarianceFreeMeans'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            'backgroundThresholdType', 'otzu', ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'numberOfGaussians', 13, ...
            'sameVariance', true, ...
            'varianceScale', 1, ...
            'updateVariance', true, ...
            'equidistantMeans', false, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'smoothingType', 'bspline', ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
        
    case 'mixtureWithPrior'
        [correctedData, estimatedBiasField, mask, infoStruct] = iic(dataS.vol, ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 2, ...
            'gmmFitType', 'mixtureWithLabelPrior', ...
            'prior', prior, ...
            'priorThreshhold', 0.99, ...
            'numberOfGaussians', [2,2,2,3], ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
        
    case 'EMwithSV'
        [correctedData, estimatedBiasField, mask, infoStruct, segmented] = iic(dataS.vol, ...
            'stepSize', [dataS.xsize, dataS.ysize, dataS.zsize], ...
            'desiredStepSize', 4, ...
            'gmmFitType', 'mixtureWithSuperVoxel', ...
            'superVoxelStepSize', 40, ...
            'smoothingDistance', 50, ...
            'smoothingRegularization', 1e-5, ...
            'showPlots', showPlots, ...
            'verbosity', 'debug');
end
